/************************************************************************
 *
 * Copyright (c) 2001
 * IONA Technologies, Inc.
 * Waltham, MA, USA
 *
 * All Rights Reserved
 *
 ************************************************************************/

#include <OBEC/CORBA.h>
#include <OBEC/Memory.h>
#include <OBEC/GlobalDestroyer.h>

#include <OBEC/CosNaming.h>
#include "server.h"
#include "FTManagement.h"

#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <sys/types.h>
#define MAX_CLIENTS 1000

float X0[MAX_CLIENTS] ;
float Y0[MAX_CLIENTS] ;
float VX[MAX_CLIENTS] ;
float VY[MAX_CLIENTS] ;

CosNaming_NamingContext nc = NULL;
CosNaming_NameComponent name_component1[1] =
{
  {"TrafficControlServer1", ""}};
CosNaming_NameComponent name_component2[1] =
{
  {"TrafficControlServer2", ""}};
CosNaming_Name name[2] =
{
  {1, 1, name_component1},
  {1, 1, name_component2}};

/* Fault tolerance */
	FTManagementModule_ReplicationManagerInterface repManager;

void
setPos (int id, float *x, float *y)
{
  *x = X0[id];
  *y = Y0[id];

}

void
setVelocity (int id, float *vx, float *vy)
{
  *vx = VX[id];
  *vy = VY[id];

}
int
resolveRepManager (FTManagementModule_ReplicationManagerInterface* rmptr, CORBA_Environment * evptr)
{

CosNaming_NameComponent repC[1] = { {"ReplicationManager", ""}};
CosNaming_Name repName = {1, 1, repC};

  do
    {

			evptr->_major = CORBA_NO_EXCEPTION;
      *rmptr = CosNaming_NamingContext_resolve (nc, &repName, evptr);
      if (evptr->_major != CORBA_NO_EXCEPTION)
	{
	  fprintf (stderr, "Could not resolve Replication Manager :%s\n", CORBA_exception_id (evptr));
	  sleep (2);
	}
    }
  while (evptr->_major != CORBA_NO_EXCEPTION);

  return EXIT_SUCCESS;

}

int
resolveServerObject (ServerModule_TrafficControl * trControlptr,
		     CORBA_Environment * evptr)
{

  static int nameId = 0;
	int flag;
	char* serverName;
	do {
	flag=0;
	fprintf(stderr,"Calling ReplicationManager::GetPrimaryServer...\n");
	evptr->_major = CORBA_NO_EXCEPTION;
	serverName = FTManagementModule_ReplicationManagerInterface_GetPrimaryServer(repManager, evptr);
	if (evptr->_major != CORBA_NO_EXCEPTION)
	{

		flag=1;
	  fprintf (stderr, "Unable to call ReplicationManager::GetPrimaryServer:%s\n", CORBA_exception_id (evptr));
	}
	else if( !strcmp(serverName,"TrafficControlServer1") ) nameId =0;
	else if( !strcmp(serverName,"TrafficControlServer2") ) nameId =1;
	else  { flag=1;fprintf(stderr, "Unknown Name given by Replication Manager : %s\n",serverName); }
	  sleep (2);
  } while(flag==1);
	fprintf(stderr,"Replication Manager tells me to switch to %s.\n",serverName);

  do
    {
  //nameId = (nameId + 1) % 2;
			
			evptr->_major = CORBA_NO_EXCEPTION;
	  fprintf (stderr, "Trying to resolve TrafficControl object:%s\n", serverName);
      *trControlptr = CosNaming_NamingContext_resolve (nc, &name[nameId], evptr);
      if (evptr->_major != CORBA_NO_EXCEPTION)
	{
	  fprintf (stderr, "Could not resolve TrafficControl object:%s\n", CORBA_exception_id (evptr));
	  sleep (2);
	}
    }
  while (evptr->_major != CORBA_NO_EXCEPTION);

  return EXIT_SUCCESS;

}

int
leave (ServerModule_TrafficControl * trControlptr, int id, CORBA_boolean errFlag, CORBA_Environment * evptr)
{
  int status = EXIT_SUCCESS;

  if (errFlag)
    printf ("Panic: Trying to Leave..\n");
  ServerModule_TrafficControl_Leave (*trControlptr, id, evptr);
  if (evptr->_major != CORBA_NO_EXCEPTION)
    {
      fprintf (stderr, "Unable to call TrafficControl_Leave , Exception:%s\n", CORBA_exception_id (evptr));
      if (evptr->_major == CORBA_SYSTEM_EXCEPTION &&
	  strcmp (CORBA_exception_id (evptr), ex_CORBA_COMM_FAILURE) == 0)
	{
	  printf ("Oops..Server crashed..Switching to Replica..\n");
	  printf ("Resolving TrafficControl object....\n");
	  status = resolveServerObject (trControlptr, evptr);
	  printf ("Resolved TrafficControl object successfully.\n");
	}
      else
	{
	  evptr->_major = CORBA_NO_EXCEPTION;
	  status = EXIT_FAILURE;
	}
    }
  if (status == EXIT_SUCCESS)
    {
      if (errFlag)
	printf ("Panic Leave was successful\n");
      else
	printf ("TrafficControl_Leave Success: Bye Bye.\n");
    }
  else
    printf ("Oops.Sorry, but I'm done!!\n");

  return status;

}


int
run (CORBA_ORB orb, int argc, char *argv[], CORBA_Environment * evptr)
{




  char *mapPos, myName[50], tmp[20];

  ServerModule_TrafficControl trControl;
  ServerModule_PositionInfo_Detail *planeSeqDetail;
  ServerModule_PositionInfo_Brief *planeSeqBrief;
  float x, y, vx, vy;
  int myid,  sleepCounter, status;
  CORBA_boolean emergencyFlag = OBEC_FALSE;
  float MAX_WIDTH = 1;



  pid_t pid = getpid ();


  OBEC_PROLOG (evptr);

  printf ("Client starting up....");
  /* Getting own name */
  sprintf (tmp, "%d", pid);
  gethostname (myName, 50);
  strcat (myName, ":");
  strcat (myName, tmp);
  printf ("My Name is %s\n", myName);

  sleepCounter = 0;
  /*
   * Get naming service
   */
  printf ("Resolving NameService....\n");
  nc = CORBA_ORB_resolve_initial_references (orb, "NameService", evptr);

  // Can't Resolve NameService
  if (evptr->_major == CORBA_SYSTEM_EXCEPTION &&
      strcmp (CORBA_exception_id (evptr), ex_CORBA_ORB_InvalidName) == 0)
    {
      fprintf (stderr, "%s: can't resolve `NameService'\n", argv[0]);
      OBEC_EPILOG_FAIL (evptr);
      evptr->_major = CORBA_NO_EXCEPTION;
      return EXIT_FAILURE;
    }

  //NameService is a nil object : Does not exist where it should be
  if (nc == NULL)
    {
      fprintf (stderr, "%s: `NameService' is a nil object reference\n", argv[0]);
      OBEC_EPILOG_FAIL (evptr);
      evptr->_major = CORBA_NO_EXCEPTION;
      return EXIT_FAILURE;
    }

  printf ("Resolved NameService successfully.\n");

// Now, try to get the interface reference for TrafficControl in module ServerModule
  printf ("Resolving Replication Manager ....\n");
  status = resolveRepManager (&repManager, evptr);
  printf ("Resolved Replication Manager successfully.\n");
  printf ("Resolving TrafficControl object....\n");
  status = resolveServerObject (&trControl, evptr);
  printf ("Resolved TrafficControl object successfully.\n");
// try  to  join
  printf ("Trying to Join..\n");

  do
    {
			evptr->_major = CORBA_NO_EXCEPTION;
      myid = ServerModule_TrafficControl_Join (trControl, myName, evptr);

      if (evptr->_major != CORBA_NO_EXCEPTION)
	{
	  fprintf (stderr, "Unable to call TrafficControl_Join , Exception:%s\n", CORBA_exception_id (evptr));

	  if (evptr->_major == CORBA_SYSTEM_EXCEPTION &&
	    strcmp (CORBA_exception_id (evptr), ex_CORBA_COMM_FAILURE) == 0)
	    {
	      printf ("Oops..Server crashed..Switching to Replica..\n");
	      printf ("Resolving TrafficControl object....\n");
	      status = resolveServerObject (&trControl, evptr);
	      printf ("Resolved TrafficControl object successfully.\n");
	    }
	}
			sleep(2);
    }
  while (evptr->_major != CORBA_NO_EXCEPTION);

  printf ("Joined successfully.\n");

//initialise Position and Velocity
  setPos (myid, &x, &y);
  setVelocity (myid, &vx, &vy);


	while(1)
    {
//calcPos
      x = (x + vx);
      y += vy;
      printf ("moving to new location (%f,%f)\n", x, y);
// TODO: check for out of range: make map from 1 to 100

      //send pos
      if (sleepCounter == 0)
	{

	      printf ("Calling TrafficControl_PositionUpdate ...\n");
	      mapPos = ServerModule_TrafficControl_PositionUpdate
		(trControl, myid, x, y, vx, vy, evptr);


	      // TODO: catch invalidpos excepttoins here

	      if (evptr->_major != CORBA_NO_EXCEPTION)
		{
		  fprintf (stderr, "Unable to call TrafficControl_PositionUpdate , Exception:%s\n", CORBA_exception_id (evptr));
		  if (evptr->_major == CORBA_SYSTEM_EXCEPTION &&
		      strcmp (CORBA_exception_id (evptr), ex_CORBA_COMM_FAILURE) == 0)
		    {
		      printf ("Oops..Server crashed..Switching to Replica..\n");
		      printf ("Resolving TrafficControl object....\n");
		      status = resolveServerObject (&trControl, evptr);
		      printf ("Resolved TrafficControl object successfully.\n");
		    }
		  else
		    {
		      status = leave (&trControl, myid, OBEC_TRUE, evptr);
		      evptr->_major = CORBA_NO_EXCEPTION;
		      return EXIT_FAILURE;
		    }
		}
	  printf ("PositionUpdate Success : I am in zone %s\n", mapPos);

	}


      // getInquiry
      if (sleepCounter == 0 && !emergencyFlag)
	{
	  printf ("Calling TrafficControl_Inquiry ...\n");
	  planeSeqBrief = ServerModule_TrafficControl_Inquiry (trControl, myid, x, y, MAX_WIDTH / 10, evptr);

	  //catchEx

	  if (evptr->_major != CORBA_NO_EXCEPTION)
	    {
	      fprintf (stderr, "Unable to get inquiry , Exception:%s\n", CORBA_exception_id (evptr));
	      if (evptr->_major == CORBA_SYSTEM_EXCEPTION &&
		  strcmp (CORBA_exception_id (evptr), ex_CORBA_COMM_FAILURE) == 0)
		{
		  printf ("Oops..Server crashed..Switching to Replica..\n");
		  printf ("Resolving TrafficControl object....\n");
		  status = resolveServerObject (&trControl, evptr);
		  printf ("Resolved TrafficControl object successfully.\n");
		}
	      else
		{
		  status = leave (&trControl, myid, OBEC_TRUE, evptr);
		  evptr->_major = CORBA_NO_EXCEPTION;
		  return EXIT_FAILURE;
		}
	    }
	  else
	    {
	      printf ("Success in TrafficControl_Inquiry: No of Planes around=%ld \n", planeSeqBrief->_length);
	      if (planeSeqBrief->_length >= 0)
		emergencyFlag = OBEC_TRUE;
	    }
	}

// getEmQuery
      else if (emergencyFlag)
	{
	  printf ("Calling TrafficControl_EmergencyInquiry ...\n");
	  planeSeqDetail = ServerModule_TrafficControl_EmergencyInquiry (trControl, myid, x, y, MAX_WIDTH / 20, evptr);

	  //catchEx
	  if (evptr->_major != CORBA_NO_EXCEPTION)
	    {
	      fprintf (stderr, "Unable to get Emergency Inquery , Exception:%s\n", CORBA_exception_id (evptr));
	      if (evptr->_major == CORBA_SYSTEM_EXCEPTION &&
		  strcmp (CORBA_exception_id (evptr), ex_CORBA_COMM_FAILURE) == 0)
		{
		  printf ("Oops..Server crashed..Switching to Replica..\n");
		  printf ("Resolving TrafficControl object....\n");
		  status = resolveServerObject (&trControl, evptr);
		  printf ("Resolved TrafficControl object successfully.\n");
		}
	      else
		{
		  status = leave (&trControl, myid, OBEC_TRUE, evptr);
		  evptr->_major = CORBA_NO_EXCEPTION;
		  return EXIT_FAILURE;
		}
	    }
	  else
	    {
	      printf ("Success in TrafficControl_EmergencyInquiry: No of Planes around=%ld \n", planeSeqDetail->_length);
	      if (planeSeqDetail->_length >= 0)
		emergencyFlag = OBEC_FALSE;
	    }
	}

      sleep (4);
      sleepCounter = (sleepCounter + 1) % 2;
    }

  printf ("Mission over: trying to TrafficControl_Leave....\n");
  status = leave (&trControl, myid, OBEC_FALSE, evptr);



  OBEC_CREATE_DESTRUCTOR (CORBA_Object_release, nc, 1, evptr);


  if (evptr->_major == CORBA_USER_EXCEPTION)
    {
      fprintf (stderr, "%s: Unexpected user exception was thrown\n", argv[0]);
      OBEC_EPILOG_FAIL (evptr);
      evptr->_major = CORBA_NO_EXCEPTION;
      return EXIT_FAILURE;
    }

  OBEC_EPILOG_SUCC (evptr);
  return EXIT_SUCCESS;
}

int
main (int argc, char *argv[])
{
  int status = EXIT_SUCCESS;
  CORBA_ORB orb;
  CORBA_Environment ev;
 int i;
for(i=0;i<MAX_CLIENTS;i++) {
  X0[i]=0.1*(i%10);
  Y0[i]=0.1*(i%10);
  if( i%10 > 4) {
  VX[i]= -0.01; 
  VY[i]= -0.01; 
  }
  else {
  VX[i]= 0.01; 
  VY[i]= 0.01; 
  }
}
  CORBA_exception_init (&ev);

  orb = CORBA_ORB_init (&argc, argv, "AirCraftClient", &ev);
  if (ev._major == CORBA_NO_EXCEPTION)
    {
      status = run (orb, argc, argv, &ev);
    }

  if (orb != NULL)
    {
      CORBA_ORB_destroy (orb, &ev);
      if (ev._major != CORBA_NO_EXCEPTION)
	{
	  fprintf (stderr, "An exception was raised during ORB destroy: %s\n",
		   CORBA_exception_id (&ev));
	  status = EXIT_FAILURE;
	}
    }

  CORBA_ORB__release (orb);

  CORBA_exception_free (&ev);

#ifdef OB_PURIFY_CLEAN
  OBEC_destroy_global_objects ();
#endif

  return status;
}
