package Server;

// TrafficControlServer.java
// Copyright and License
import ServerModule.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import org.omg.CORBA.*;
import org.omg.PortableServer.*;

import java.util.Properties;
import java.math.*;

class TrafficControlServerImpl extends TrafficControlPOA
{
  private ORB orb=null;
  private Map map=null;
  
  private static const int totalClients = 10; \\we can change this later to be dynamic
  private Boolean[totalClients] planeID;
   
  private Plane[] planesList ; \\ array of plane data structures, basically all the 
							    \\planes handled by the system
  
  static init_IdArray ()
  {
	for (int i=0; i<totalClients; i++)
		planeID[i] = false;
  
  }
  
  public boolean Init(ORB orb_val)
  {
    orb = orb_val;
    map=new Map();
    boolean chk=map.load();
    assert(chk);
    return true;     
  }
 
  //Clients: Planes
  public int join() 
  {
	int counter = 0 ;
	
	synchronize (planeID)
	{
		for(int counter = 0; counter<totalClients; counter++) 
		{
			if (planeID[counter] != true)
			{
				planeID[counter] = true;
				return counter ;
			}	
		}		
	}	
	System.out.println("Maximum Plane Handing Capacity reached...
						No more planes can be handled !!! So if you take off you die\n");
  }


  //Clients: Planes
  public boolean Leave(int id) throws ServerModule.invalid_plane_id
  {
	if ((0 <= id )&& (id < totalClients) && (planeID[id] == true))
		{ 
			planeID[id] = false;
			return true;
		}
	else throw new invalid_plane_id(); 
	return false; 
  }
  
  
  //Clients: Planes
  public String PositionUpdate(int id, float x, float y, float vx, float vy) 
      throws  ServerModule.invalid_plane_id,ServerModule.invalid_coordinate
  {

      String towerName = null;
      try {
          towerName = map.lookup(x,y);

          if (towerName.equals("unknown"))
                         throw new invalid_coordinate();

          //Backup.positionBackup (id, x, y, vx, vy);

          db.positionSave(id, x, y, vx, vy); \**********************\

          //need to have something which throws invalid_plane_id exception
      } catch (Exception e) {
          e.printStackTrace();  //To change body of catch statement use Options | File Templates.
          return "Exception";
      }

      return towerName;
  }

  
  //Clients: Planes
  public ServerModule.PlanePosition_Brief[] Inquiry(int id, float x, float y,
      int radius1) throws ServerModule.invalid_plane_id, ServerModule.invalid_coordinate
  {
   
	//find the distance between the given plane and other planes by distance formula 
	// and then see if it is less than or equal to radius1
	//We return the array containing the ids of all the planes within that radius
	
	ServerModule.PlanePosition_Brief[] returnArray ;
	
	float x1;
	float y1;
	float distance = 0.0;
		
	for (int i = 0; i < planesList.length; i++)
	{
		x1 = planesList[i].x;
		y1 = planesList[i].y;
		
		 distance = sqrt(sq(x - x1) + sq(y - y1)); \\ Dont know the exact syntax for the square root
													\\and squaring functions in Java
		if (distance < float(radius1))
		{
			returnArray = new ServerModule.PlanePosition_Brief[] returnArray;
			returnArray[returnArray.length].id = planesList[i].id;
			returnArray[returnArray.length].x = planesList[i].x;
			returnArray[returnArray.length].y = planesList[i].y;
		}
			
	} 
   return returnArray;
  }

  //Clients: Planes
  public ServerModule.PlanePosition_Detail[] EmergencyInquiry(int id, float x,
      float y, int radius2) throws ServerModule.invalid_plane_id, ServerModule.invalid_coordinate
  {
    return null;
  }

  // implement shutdown() method
  public void shutdown()
  {
    orb.shutdown(false);
  }
}

public class TrafficControlServer
{

  public static void main(String args[])
  {
    try
    {
      // create and initialize the ORB
      ORB orb = ORB.init(args, null);

      // get reference to rootpoa & activate the POAManager
      POA rootpoa = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));
      rootpoa.the_POAManager().activate();

      // create servant and register it with the ORB
      TrafficControlServerImpl ATCImpl = new TrafficControlServerImpl();
      //Init
      ATCImpl.Init(orb);
      
      // //Initialise the ID array
      init
      
      // get object reference from the servant
      org.omg.CORBA.Object ref = rootpoa.servant_to_reference(ATCImpl);
      TrafficControl href = TrafficControlHelper.narrow(ref);

      // get the root naming context
      // NameService invokes the name service
      org.omg.CORBA.Object objRef =
          orb.resolve_initial_references("NameService");
      // Use NamingContextExt which is part of the Interoperable
      // Naming Service (INS) specification.
      NamingContextExt ncRef = NamingContextExtHelper.narrow(objRef);

      // bind the Object Reference in Naming
      String name = "TrafficControlServer";
      NameComponent path[] = ncRef.to_name(name);
      ncRef.rebind(path, href);

      System.out.println("TrafficControlSever ready and waiting ...");

      // wait for invocations from clients
      orb.run();
    }

    catch (Exception e)
    {
      System.err.println("ERROR: " + e);
      e.printStackTrace(System.out);
    }

    System.out.println("TrafficControlSever Exiting ...");

  }
}