package FTManagement;

// TrafficControlServer.java
// Copyright and License

import FTManagementModule.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import org.omg.CORBA.*;
import org.omg.CORBA.ORBPackage.InvalidName;
import org.omg.PortableServer.*;
import java.util.Timer;
import java.util.TimerTask;

class ExecutiveCheckTask extends TimerTask {
  private ExecutiveImpl m_EXECImpl;
  ExecutiveCheckTask(ExecutiveImpl EXECImpl) {
    m_EXECImpl = EXECImpl;
  }
  public void run() {
    m_EXECImpl.run();
  }
}


public class Executive {

    /**
     * This message prints out an exception to user and exits gracefully
     *
     */
    public static void exceptionHelper(String message, Exception e, boolean debug) {
        System.err.println(message);
        if (debug) e.printStackTrace();
        System.out.println("Executive Exiting ...");
        System.exit(1);
    }

    /**
     *   Returns true if the given string is contained within the argument list.
     */
    protected static boolean argHelper(String lookFor, String args[])
    {
        boolean toReturn = false;
        for (int i = 0; i < args.length; i++) {
            String arg = args[i];
            if (arg.equals(lookFor))
                toReturn = true;
        }
        return toReturn;
    }
    protected static String argGetAll(String args[])
    {
      String arg_string=new String();
      for (int i = 0; i < args.length; i++) {
          String arg = args[i];
          arg_string+=arg;
          arg_string+=" ";
      }
      return arg_string;
    }
   protected static String getServerID(String args[])
    {
        String serverID = "";

        for (int i = 0; i < args.length; i++) {
            String arg = args[i];
            if (arg.equals("-id"))
                serverID = args[i+1];
        }
        System.out.println("Server ID = "+serverID);
        return serverID;
    }

    public static void main(String args[]) {

        if (argHelper("-help",args))
        {
            System.out.println("Usage: java FTManagement.Executive [-d] -id n");
            System.out.println("            -d  debug");
            System.out.println("            -id [1|2] server name");
        }

        boolean debugMode = false;
        if (argHelper("-d", args)) debugMode = true;


        boolean debug = false;
        try {
          //get server id
          String serverID = getServerID(args);
          if (serverID.equals("")) {
            System.out.println("Please use -id n to specify the server name");
            return;
          }
          // create and initialize the ORB

          ORB orb = ORB.init(args, null);

              System.out.println("ORB initialized");

          // get reference to rootpoa & activate the POAManager
          POA rootpoa = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));

          System.out.println("reference to root POA...");

          rootpoa.the_POAManager().activate();

          System.out.println("POAManager activated");

          // create servant and register it with the ORB
          int serverid=Integer.parseInt(serverID.trim());
          ExecutiveImpl ExecImpl = new ExecutiveImpl(serverid, orb, debugMode,argGetAll(args));

              System.out.println(" Servant is created and registered with the ORB");

          // get object reference from the servant
          org.omg.CORBA.Object ref = rootpoa.servant_to_reference(ExecImpl);
              System.out.println(" Servant references object");
          ExecutiveInterface href = ExecutiveInterfaceHelper.narrow(ref);

          // get the root naming context
          // NameService invokes the name service

          org.omg.CORBA.Object objRef = null;
          try {
            objRef = orb.resolve_initial_references("NameService");
          }
          catch (InvalidName invalidName) {
            exceptionHelper(
                "Could not resolve initial ORB reference. Is your naming service started?",
                invalidName, debugMode);
          }
          catch (org.omg.CORBA.COMM_FAILURE orbTrouble) {
            exceptionHelper(
                "Could not resolve initial ORB reference. Is your naming service started?",
                orbTrouble, debugMode);
          }
               System.out.println("Naming service is invoked ");

          // Use NamingContextExt which is part of the Interoperable
          // Naming Service (INS) specification.
          //NamingContextExt ncRef = NamingContextExtHelper.narrow(objRef);

          NamingContext ncRef = NamingContextHelper.narrow(objRef);

          NameComponent nc = new NameComponent("Executive" + serverID, "");
          NameComponent path[] = {
              nc};
          ncRef.rebind(path, href);

          System.out.println("Executive " + serverID + " ready and waiting ...");

          //start the checking
          System.out.println("Start the checking");

          Timer timer = new Timer();
          timer.schedule(new ExecutiveCheckTask(ExecImpl), 1*1000,1*1000); //every 1s

          // wait for invocations from clients
         System.out.println("Waiting for invocations from clients");

          orb.run();

          timer.cancel();
        }
        catch (Exception e) {
          exceptionHelper("Unexpected error has occurred.", e, debug);
        }

        System.out.println("Executive exiting");

      }
}