package FTManagement;
import FTManagementModule.ExecutiveInterfacePOA;
import FTManagementModule.ReplicationManagerInterface;
import FTManagementModule.ReplicationManagerInterfaceHelper;
import FTManagementModule.SERVER_DOWN;
import org.omg.CORBA.Object;
import org.omg.CORBA.ORB;
import ServerModule.TrafficControl;
import ServerModule.TrafficControlHelper;
import java.io.*;
import org.omg.CORBA.ORBPackage.*;
import org.omg.CosNaming.*;
public class ExecutiveImpl extends ExecutiveInterfacePOA
{
  private String  command_line;
  private StreamGobbler errorGobbler = null;
  private StreamGobbler outputGobbler = null;
  private int     ATC_failure_count=1;
  private int     server_id;
  private boolean mDebugMode=false;
  private ORB             orb=null;
  private ReplicationManagerInterface replication_manager=null;
  private TrafficControl   traffic_control_server=null;



  public boolean is_alive() {
    if (traffic_control_server==null)
      traffic_control_server=GetTrafficControlServerInterface();
    if (traffic_control_server!=null)
      {
        try {
          return traffic_control_server.is_alive();
        }
        catch (Exception ex) {

          //System.out.println("Starting is_alive trace...");

          trace("server "+server_id+"is_alive failed");
          traffic_control_server=null;
          return false;
        }
      }
    return true;
  }

  public boolean become_primary() {
    System.out.println("Server became primary");
    if (traffic_control_server == null)
      return false;
    else {System.out.println("Server is going to become primary");
      try {
        return traffic_control_server.become_primary();
      }
      catch (Exception ex) {

        traceError("server "+server_id+"become_primary failed",ex);

         System.out.println("Server fails to become primary");
        //invalid this reference.
        traffic_control_server=null;
        return false;
      }
    }
  }



  ExecutiveImpl(int serverid, ORB orb_env, boolean debugmode, String args) {
    this.command_line=args;
    this.server_id=serverid;
    this.orb=orb_env;
    mDebugMode = debugmode;
//    replication_manager=GetReplicationManagerInterface();
//    traffic_control_server= GetTrafficControlServerInterface();
  }
  ReplicationManagerInterface GetReplicationManagerInterface()
  {

    try {
      // get the root naming context
      org.omg.CORBA.Object objRef = orb.resolve_initial_references(
          "NameService");
      // Use NamingContextExt instead of NamingContext. This is
      // part of the Interoperable naming Service.
      NamingContextExt ncRef = NamingContextExtHelper.narrow(objRef);
      // resolve the Object Reference in Naming
      String name = "ReplicationManager";
      ReplicationManagerInterface rmi = ReplicationManagerInterfaceHelper.
          narrow(ncRef.resolve_str(name));
      return rmi;
    }
    catch (Exception ex) {
      trace("GetReplicationManagerInterface failed, means the replication manager is not ready?");
      return null;
    }
  }
  TrafficControl GetTrafficControlServerInterface()
  {
    try {
      // get the root naming context
      Object objRef = null;
      try {
        objRef = orb.resolve_initial_references(
            "NameService");
      }
      catch (Exception ex1) {
        trace("can not contact with NameService"+ex1.getMessage());
      }

      // resolve the Object Reference in Naming
      String name = "TrafficControlServer"+server_id;
      NamingContext ncRef = NamingContextHelper.narrow(objRef);
      NameComponent nc[]={new NameComponent(name,"")};
      
      TrafficControl ATCinterface = TrafficControlHelper.narrow(ncRef.resolve(nc));
      return ATCinterface;
    }
    catch (Exception ex) {
      trace("GetTrafficControlServerInterface failed, means airtraffic server crashed");
      return null;
    }
  }
  public  void run()
  {

  //restart the ATC server failed for continuous 10times(10s)
     if (traffic_control_server==null)
     {
         System.out.println("Restart the ATC failed" );

         //could get the interface doesn't mean the server is working
         traffic_control_server=GetTrafficControlServerInterface();
         if (traffic_control_server==null)
           ATC_failure_count++;
     }
     //restart the server if neccesary
     System.out.println("Restarting server");
     if (ATC_failure_count>=10)
     {
           ReStartTrafficControlServer();
           ATC_failure_count= 1;
     }

     if (replication_manager==null)
     {
        replication_manager=GetReplicationManagerInterface();
     }
     //maybe notify the ReplicationManager
      System.out.println("Notifying the Replication Management that server crashed");
     if (traffic_control_server!=null)
     {
        try {
          boolean status = traffic_control_server.is_alive();
          ATC_failure_count= 1;
        }
        catch (Exception ex) {
          ATC_failure_count++;
          trace("AirTrafficControlServer "+server_id+" crashed?");
          traffic_control_server=null;
          try {
            if (replication_manager!=null)
              replication_manager.EventNotify(server_id, SERVER_DOWN.value);
          }
          catch (Exception ex1) {
            replication_manager=null;
             System.out.println("Notification of crashed server failed");
          }
        }
     }
  }
  private boolean ReStartTrafficControlServer()
  {
    //then redirect the
    System.err.println("ReStartTrafficControlServer\n");
     System.out.println("Restarting ATC server");
    try {
      Runtime rt = Runtime.getRuntime();
      Process proc = rt.exec("java "+ "Server/TrafficControlServer " + command_line);
      // any error message?
      this.errorGobbler = new StreamGobbler(proc.getErrorStream(), "ERROR");
      // any output?
      this.outputGobbler = new StreamGobbler(proc.getInputStream(), "OUTPUT");
      // kick them off
      errorGobbler.start();
      outputGobbler.start();
      // any error???
      //int exitVal = proc.waitFor();
      //System.out.println("ExitValue: " + exitVal);
    }
    catch (IOException ex) {
      traceError("ReStartTrafficControlServer failed\n",ex);
       System.out.println("Factory cannot conduct process");
    }
    return true;
  }
  private void trace(String message) {
     System.out.println("Exception at factory conducting process");
      if (mDebugMode)
          System.out.println(message);
  }
  /**
   * Prints exception message. Only prints stack trace if debug mode is on.
   */
  private void traceError(String message, Exception e) {
      System.err.println(message);
      if (mDebugMode)
          e.printStackTrace();
  }
}

class StreamGobbler extends Thread
{
    InputStream is;
    String type;
    OutputStream os;

    StreamGobbler(InputStream _is, String _type)
    {
        this(_is, _type, null);
    }

    StreamGobbler(InputStream _is, String _type, OutputStream redirect)
    {
        this.is = _is;
        this.type = _type;
        this.os = redirect;
    }

    public void run()
    {
       System.out.println("Running?");

        try
        {
            PrintWriter pw = null;
            if (os != null)
                pw = new PrintWriter(os);

            InputStreamReader isr = new InputStreamReader(is);
            BufferedReader br = new BufferedReader(isr);
            String line=null;
            while ( (line = br.readLine()) != null)
            {
                if (pw != null)
                    pw.println(line);
                System.out.println(type + ">" + line);
            }
            if (pw != null)
                pw.flush();
        } catch (IOException ioe)
            {
            ioe.printStackTrace();
            }
    }
}
