package FTManagement;

import FTManagementModule.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import org.omg.CORBA.*;
import org.omg.CORBA.ORBPackage.InvalidName;
import org.omg.PortableServer.*;
import java.util.TimerTask;
import java.util.Timer;

class CheckTask extends TimerTask {
  private ReplicationManagerImpl m_RMImpl;
  CheckTask(ReplicationManagerImpl RMImpl) {
    m_RMImpl = RMImpl;
  }

  public void run() {
    m_RMImpl.run();
  }
}

public class ReplicationManager{

    /**
     * This message prints out an exception to user and exits gracefully
     *
     */
    public static void exceptionHelper(String message, Exception e, boolean debug) {
        System.err.println(message);
        if (debug) e.printStackTrace();
        System.out.println("ReplicationManager Exiting ...");
        System.exit(1);
    }

    /**
     *   Returns true if the given string is contained within the argument list.
     */
    protected static boolean argHelper(String lookFor, String args[])
    {
        boolean toReturn = false;
        for (int i = 0; i < args.length; i++) {
            String arg = args[i];
            if (arg.equals(lookFor))
                toReturn = true;
        }
        return toReturn;
    }
    public static void main(String args[]) {

        if (argHelper("-help",args))
        {
            System.out.println("Usage: java FTManagement.ReplicationManager [-d] -id n");
            System.out.println("            -d  debug");
        }

        boolean debugMode = false;
        if (argHelper("-d", args)) debugMode = true;


        boolean debug = false;
        try {
            // create and initialize the ORB
            ORB orb = ORB.init(args, null);

            // get reference to rootpoa & activate the POAManager
            POA rootpoa = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));
            rootpoa.the_POAManager().activate();

            // create servant and register it with the ORB
            ReplicationManagerImpl RMImpl = new ReplicationManagerImpl(debugMode);
            //Init
            RMImpl.Init(orb);

            // get object reference from the servant
            org.omg.CORBA.Object ref = rootpoa.servant_to_reference(RMImpl);
            ReplicationManagerInterface href = ReplicationManagerInterfaceHelper.narrow(ref);

            // get the root naming context
            // NameService invokes the name service

            org.omg.CORBA.Object objRef = null;
            try {
                objRef = orb.resolve_initial_references("NameService");
            } catch (InvalidName invalidName) {
                exceptionHelper("Could not resolve initial ORB reference. Is your naming service started?", invalidName, debugMode);
            } catch (org.omg.CORBA.COMM_FAILURE orbTrouble) {
                exceptionHelper("Could not resolve initial ORB reference. Is your naming service started?", orbTrouble, debugMode);
            }

            // Use NamingContextExt which is part of the Interoperable
            // Naming Service (INS) specification.
            //NamingContextExt ncRef = NamingContextExtHelper.narrow(objRef);

            NamingContext ncRef = NamingContextHelper.narrow(objRef);

            NameComponent nc = new NameComponent("ReplicationManager","");
            NameComponent path[] = {nc};
            ncRef.rebind(path, href);

            System.out.println("ReplicationManager ready and waiting ...");

            //start the checking
            Timer timer = new Timer();
            timer.schedule(new CheckTask(RMImpl), 1*1000,1*1000); //every 1s

            System.out.println("Replication Manamger running");
            // wait for invocations from clients
            orb.run();

            timer.cancel();
        } catch (Exception e) {
            exceptionHelper("Unexpected error has occurred.",e,debug);
        }

        System.out.println("ReplicationManager Exiting ...");

    }

}