package FTManagement;
import FTManagementModule.ReplicationManagerInterfacePOA;
import FTManagementModule.SERVER_ALIVE;
import FTManagementModule.SERVER_DOWN;
import FTManagementModule.ExecutiveInterface;
import FTManagementModule.ExecutiveInterfaceHelper;
import ServerModule.TrafficControl;
import ServerModule.TrafficControlHelper;
import org.omg.CosNaming.*;
import org.omg.CORBA.Object;
import org.omg.CORBA.ORB;
import FTManagementModule.invalid_event;
import org.omg.CORBA.ORBPackage.*;
class ReplicationManagerImpl extends ReplicationManagerInterfacePOA{
  private ORB orb;
  private boolean mDebugMode;
  private int server1_statue=SERVER_DOWN.value; //down
  private int server2_statue=SERVER_DOWN.value; //down
  private int primary_id=0; //none is primary
  //ExecutiveInterface   executive1=null;
  //ExecutiveInterface   executive2=null;
  TrafficControl executive1=null;
  TrafficControl executive2=null;


  public synchronized boolean EventNotify (int serverid, int event)
      throws FTManagementModule.invalid_event
  {

    System.out.println("ALARM: server "+serverid+
                       " is "+(event==SERVER_DOWN.value?"DOWN":"ALIVE"));

      if (serverid!=1&&serverid!=2)
        throw (new FTManagementModule.invalid_event());
      if (event!=SERVER_ALIVE.value&&event!=SERVER_DOWN.value)
        throw (new FTManagementModule.invalid_event());
      //first update the status
      if (serverid == 1) {
        server1_statue = event;
      }
      if (serverid == 2) {
        server2_statue = event;
      }
      //check if the primary will be changed
      if (primary_id==0||
          (server1_statue==SERVER_DOWN.value&&primary_id==1)||
          (server2_statue==SERVER_DOWN.value&&primary_id==2)
          )
      {
          if (server1_statue == SERVER_ALIVE.value&&executive1!=null) {
             System.err.println("primary change from: "+primary_id+"=>1 \n");
            try {
              if (executive1.become_primary())
                primary_id=1;
            }
            catch (Exception ex) {
                System.err.println("primary change failed: "+primary_id+"=>1 \n");
                primary_id=0;
                //if error happens, recheck its statues.
                //TODO: have a check of the possiblity of deadlock
                run();
            }
          }
          else if (server2_statue == SERVER_ALIVE.value&&executive2!=null){
             System.err.println("primary change from: "+primary_id+"=>2 \n");
             try {
              if (executive2.become_primary())
                primary_id=2;
             }
             catch (Exception ex1) {
               System.err.println("primary change failed: "+primary_id+"=>1 \n");
               primary_id=0;
               //if error happens, recheck its statues.
               //TODO: have a check of the possiblity of deadlock
               run();
             }
          }
          else
            {
               System.err.println("no primary available \n");
               primary_id=0;
            }

      }
      return true;
  }
  ReplicationManagerImpl(boolean debugmode)
  {
    mDebugMode=debugmode;
  }
  public boolean Init(ORB orb_env)
  {
      //
      this.orb = orb_env;
      //get reference of the executives
      executive1=GetTrafficControlServerReference(1);
      executive2=GetTrafficControlServerReference(2);
      return true;
  }
  private TrafficControl GetTrafficControlServerReference(int serverid)
  {
    try {
      // get the root naming context
      Object objRef = null;
      try {
        objRef = orb.resolve_initial_references("NameService");
      }
      catch (Exception ex1) {
        trace("can not contact with NameService"+ex1.getMessage());
      }

      // Use NamingContextExt instead of NamingContext. This is
      // part of the Interoperable naming Service.
      NamingContext ncRef = NamingContextHelper.narrow(objRef);

      // resolve the Object Reference in Naming
      String name = "TrafficControlServer"+serverid;
      NameComponent nc[]={new NameComponent(name,"")};
      
      Object trafficobject=null;
      try {
         trafficobject = ncRef.resolve(nc);
      }
      catch (Exception ex2) {
        trace("Exception: can not resolve "+name);
      }
      TrafficControl executive = null;
      try {
        executive = TrafficControlHelper.narrow(trafficobject);
      }
      catch (Exception ex3) {
        trace("Exception: can not resolve "+name);
      }
      return executive;
    }
    catch (Exception ex) {
      trace("[current primary:"+primary_id+"]"+"GetTrafficControlServerReference failed, the server machine "+ serverid+ " not started?");
      trace(ex.getMessage());
      //traceError("GetExecutiveReference failed, the traffic control machine notstarted?",ex);
      return null;
    }
  }
  private int CheckServerStatus(int serverid)
  {
     TrafficControl executive=(serverid==1)?executive1:executive2;
     boolean status=false;
     try {
       status=executive.is_alive();
       return status==true?SERVER_ALIVE.value:SERVER_DOWN.value;
     }
     catch (Exception ex) {
       trace("[current primary:"+primary_id+"]"+"is_alive() for TrafficControlServer "+serverid+" failed");
       return SERVER_DOWN.value;
     }
  }
  public  void run()
  {
    if (executive1==null||server1_statue==SERVER_DOWN.value)
    {
        executive1=GetTrafficControlServerReference(1);
    }
    if (executive2==null||server2_statue==SERVER_DOWN.value)
    {
        executive2=GetTrafficControlServerReference(2);
    }
     //periods check the status of the executives
     if (executive1!=null)
     {
         int new_server1_statue=CheckServerStatus(1);
         if (new_server1_statue!=server1_statue)
         {
              try {
                EventNotify(1, new_server1_statue);
              }
              catch (invalid_event ex) {
                //impossible
                System.out.println("Invalid event 1");
              }
         }
      }
    if (executive2!=null)
    {
        int new_server2_statue=CheckServerStatus(2);
        if (new_server2_statue!=server2_statue)
      {
          try {
            EventNotify(2, new_server2_statue);
          }
          catch (invalid_event ex1) {
            //impossible
             System.out.println("Invalid event 2");
          }
      }
     }
  }
  public synchronized String GetPrimaryServer ()
  {
    //first check all the server status
    run();
    System.out.println("Client contacted me and GetPrimaryServer returns: "+primary_id);
    if (primary_id != 0)
      return "TrafficControlServer" + primary_id;
    else
      return "";
  }
  private void trace(String message) {
      if (mDebugMode)
          System.out.println(message);
  }

  /**
   * Prints exception message. Only prints stack trace if debug mode is on.
   */
  private void traceError(String message, Exception e) {
      System.err.println(message);
      if (mDebugMode)
          e.printStackTrace();
  }
};