/** 
 *	TrafficControlServer.java
 *	
 *	Created by Dehua Zhang
 *	Modified by Dehua Zhang
 *	Modified by Greg Bylenok
 *	Modified by Deepti Srivastava
 *
 */
package Server;

// TrafficControlServer.java
// Copyright and License

import ServerModule.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import org.omg.CORBA.*;
import org.omg.CORBA.ORBPackage.InvalidName;
import org.omg.PortableServer.*;





public class TrafficControlServer {

    /**
     * This message prints out an exception to user and exits gracefully
     *
     */
    public static void exceptionHelper(String message, Exception e, boolean debug) {
        System.err.println(message);
        if (debug) e.printStackTrace();
        System.out.println("TrafficControlSever Exiting ...");
        System.exit(1);
    }

    /**
     *   Returns true if the given string is contained within the argument list.
     */
    protected static boolean argHelper(String lookFor, String args[])
    {
        boolean toReturn = false;
        for (int i = 0; i < args.length; i++) {
            String arg = args[i];
            if (arg.equals(lookFor))
                toReturn = true;
        }
        return toReturn;
    }

    /**
     *   Returns the Server ID number from the command line to be assigned
     *	 to each replica of server object.
     */
   protected static String getServerID(String args[])
    {
        String serverID = "";
        for (int i = 0; i < args.length; i++) {
            String arg = args[i];
            if (arg.equals("-id"))
                serverID = args[i+1];
        }
        return serverID;
    }

    public static void main(String args[]) {

        if (argHelper("-help",args))
        {
            System.out.println("Usage: java Server.TrafficControlServer [-d] -id n");
            System.out.println("            -d  debug");
            System.out.println("            -id [1|2] server name");
        }

        boolean debugMode = false;
        if (argHelper("-d", args)) debugMode = true;


        boolean debug = false;
        try {
            // create and initialize the ORB
            ORB orb = ORB.init(args, null);

            // get reference to rootpoa & activate the POAManager
            POA rootpoa = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));
            rootpoa.the_POAManager().activate();

            // create servant and register it with the ORB
            TrafficControlServerImpl ATCImpl = new TrafficControlServerImpl();
            //Init
            ATCImpl.Init(orb, debugMode);

            // get object reference from the servant
            org.omg.CORBA.Object ref = rootpoa.servant_to_reference(ATCImpl);
            TrafficControl href = TrafficControlHelper.narrow(ref);

            // get the root naming context
            // NameService invokes the name service
            org.omg.CORBA.Object objRef = null;
            try {
                objRef = orb.resolve_initial_references("NameService");
            } catch (InvalidName invalidName) {
                exceptionHelper("Could not resolve initial ORB reference. Is your naming service started?", invalidName, debugMode);
            } catch (org.omg.CORBA.COMM_FAILURE orbTrouble) {
                exceptionHelper("Could not resolve initial ORB reference. Is your naming service started?", orbTrouble, debugMode);
            }

            // Use NamingContextExt which is part of the Interoperable
            // Naming Service (INS) specification.
            //NamingContextExt ncRef = NamingContextExtHelper.narrow(objRef);

            NamingContext ncRef = NamingContextHelper.narrow(objRef);

            // bind the Object Reference in Naming Service
            // String name = "TrafficControlServer";
            //NameComponent path[] = ncRef.to_name(name);
            
            //get server id for this particular replica of the server object 			
            String serverID = getServerID(args);
            if (serverID.equals(""))
            {
                System.out.println("Please use -id flag to specify the server name");
                return;
            }
            NameComponent nc = new NameComponent("TrafficControlServer" + serverID, "");
            NameComponent path[] = {nc};
            ncRef.rebind(path, href);

            System.out.println("TrafficControlServer "+serverID+" ready and waiting ...");

            // wait for invocations from clients
            orb.run();
            System.out.println("TrafficControlSever "+serverID+" Exiting ...");
        } catch (Exception e) {
            exceptionHelper("Unexpected error has occurred.",e,debug);
        }

    }
}