package Server.db;

/* Using analogy with DBTower */

import Server.Plane;

import java.sql.*;
import java.util.Vector;

public class DBPlane {

/* inserting id of an airplane, its coordinates, and velocities by ort and ordinate */
    private static String LOAD_SQL = "SELECT UID, POS_X, POS_Y, V_X, V_Y FROM PLANE";
    private static final String INSERT_SQL_PREFIX = "INSERT INTO PLANE(NAME) VALUES ('";
    private static final String INSERT_SQL_SUFFIX = "')";
    private static final String INSERT_RECOVERY_SQL = "SELECT UID FROM PLANE WHERE NAME=?";
    private static final String UPDATE_SQL = "UPDATE PLANE SET POS_X =?, POS_Y=?, V_X=?, V_Y=? WHERE UID=?";
    private static final String DELETE_SQL = "DELETE FROM PLANE WHERE UID=?";

    /**
     * Loads all the planes from the database
     *
     * @param aConn
     * @return
     * @throws SQLException
     */
    protected static Plane[] loadAll(Connection aConn) throws SQLException {
        Statement sqlStmt = aConn.createStatement();
        ResultSet rs = sqlStmt.executeQuery(LOAD_SQL);

        Vector results = new Vector();

        while (rs.next()) {

            Plane position_log = null;

            int id = rs.getInt(1);
            float x = rs.getFloat(2);

            if (rs.wasNull())
            {
                //this plane has joined but does not look to have a position yet
                position_log = new Plane(id);
            }
            else
            {
                float y = rs.getFloat(3);
                float vx = rs.getFloat(4);
                float vy = rs.getFloat(5);
                position_log = new Plane(id, x, y, vx, vy);
            }

            results.addElement(position_log);
        }

        return (Plane[]) results.toArray(new Plane[0]);

    }

    /**
     *
     * @param aConn
     * @param aName Name of the plane. Likely Host+processID
     * @return the uid of the row just inserted. -1 indicates an error
     * @throws SQLException
     */
    protected static int insertPlane(Connection aConn, String aName) throws SQLException {

        int toReturn = -1;

        Statement insert = aConn.createStatement();

        String sql = INSERT_SQL_PREFIX + aName+ INSERT_SQL_SUFFIX;

        try
        {
            insert.execute(sql, Statement.RETURN_GENERATED_KEYS);
            ResultSet rs = insert.getGeneratedKeys();

            while (rs.next())
            {
                toReturn = rs.getInt(1);
            }

        }
        catch (SQLException e)
        {
            //detect a second, duplicate insert here. If plane is inserted a second time, then
            // a SQLException with a SQLState value of S1009 (invalid argument) will be thrown
            // by MySQL. That's our cue that it is a duplicate. If so, then read existing ID from
            // database.
            String sqlState = e.getSQLState();

            if ("S1009".equals(sqlState))
            {
                PreparedStatement select=aConn.prepareStatement(INSERT_RECOVERY_SQL);
                select.setString(1,aName);
                ResultSet rs = select.executeQuery();

                if (rs.next())
                {
                    toReturn = rs.getInt(1);
                }
                else
                {
                    throw e;
                }
            }
            else
            {
                throw e;
            }
        }

        return toReturn;
    }



    /**
     * Removes the airplane from the database
     *
     * @param aConn - a connection to the database
     * @param uid id of airplane to remove
     * @throws SQLException
     */
    protected static void removePlane(Connection aConn, int uid) throws SQLException {
        PreparedStatement deleteStmt = aConn.prepareStatement(DELETE_SQL);
        deleteStmt.setLong(1, uid);
        deleteStmt.executeUpdate();

    }

    /**
     * Updates the position of an airplane in the database
     *
     * @param aConn - a connection to the database
     * @param somePosition - one of the airplane position
     * @throws SQLException
     */
    protected static void updatePlane(Connection aConn, Plane somePosition) throws SQLException {
        PreparedStatement updateX = aConn.prepareStatement(UPDATE_SQL);
        updateX.setFloat(1, somePosition.getX());
        updateX.setFloat(2, somePosition.getY());
        updateX.setFloat(3, somePosition.getVx());
        updateX.setFloat(4, somePosition.getVy());
        updateX.setLong(5, somePosition.getId());

        updateX.executeUpdate();

    }


}
