package Server.test;

import org.omg.CORBA.ORB;
import org.omg.CosNaming.NamingContextExt;
import org.omg.CosNaming.NamingContextExtHelper;
import ServerModule.TrafficControl;
import ServerModule.TrafficControlHelper;
import ServerModule.PlanePosition_Brief;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;


public class TestClient {

    static TrafficControl tfImpl;


     /**
     * This method reads a single character from the keyboard and
     * returns it to the caller.
     */
    public static char keyboardReadChar() {

        BufferedReader myReader = new BufferedReader(new InputStreamReader(
                                  System.in));

        char charItem = ' ';

        try {
            charItem = (char) myReader.read();
        } catch (IOException err) {
            System.out.println("Read Error in keyboardReadChar method");
            err.printStackTrace();
        }

        return charItem;
    }

        /**
         * Sets up the CORBA stuff
         * @throws Exception - any exception
         */
        public static void setupORB(String[] args) throws Exception
        {
                // create and initialize the ORB
                ORB orb = ORB.init(args,null);

                // get the root naming context
                org.omg.CORBA.Object objRef = orb.resolve_initial_references("NameService");

                // Use NamingContextExt instead of NamingContext. This is
                // part of the Interoperable naming Service.
                NamingContextExt ncRef = NamingContextExtHelper.narrow(objRef);

                // resolve the Object Reference in Naming
                String name =  "TrafficControlServer1";
                tfImpl = TrafficControlHelper.narrow(ncRef.resolve_str(name));

        }


    /**
     * Usage:
     *
     * java TestClient
     * or
     * java TestClient -name <client_name>
     *
     * If name arguments are not specified, then it defaults to "plane_TestClient"
     *
     */
    public static void main(String args[]) {
        try {

            String name = "plane_TestClient";

            //handle input arguments
            if (args.length == 2)
            {
                String firstArg = args[0];
                if ("-name".equals(firstArg))
                {
                    name = args[1];
                }
            }

            //hardcode the orb settings for now.
            String[] orbargs = new String[]{"-ORBInitialPort","1500"};
            setupORB(orbargs);

            System.out.println("Obtained a handle on server object: " + tfImpl);

           //Test Join //////////////////////////////////////

           int id=tfImpl.Join(name);

            System.out.println("Plane '"+name+"' has successfully joined.");
            System.out.println("Press <ENTER> to continue...");
            keyboardReadChar();

            // Test PositionUpdate //////////////////////////////////////

            String tower = tfImpl.PositionUpdate(id, 0.1f, 0.2f, 0.3f, 0.4f);
            System.out.println("The plane is in tower "+tower+"'s territory.");

            // Test Inquiry /////////////////////////////////////////////
            //plane inquires for everything within 1 unit away.
            int radius = 1;
            PlanePosition_Brief[] planes = tfImpl.Inquiry(id, 0.25f, 0.25f, radius);
            System.out.println(planes.length +" planes found within "+radius+" units of plane "+id);
            //show the planes information
            if (planes.length>0)
            {
               for (int i=0;i<planes.length;i++)
               {
                  System.out.println("id:"+planes[i].id+"  x:"+planes[i].x+"  y:"+planes[i].y);
               }
            }
            ///////////////////////////////////////////////////////////////
            //Test Leave //////////////////////////////////////

            System.out.println("Press <ENTER> to leave...");
            keyboardReadChar();

            boolean leave=tfImpl.Leave(id);

            //tfImpl.shutdown();

        } catch (Exception e) {
            System.out.println("ERROR : " + e);
            e.printStackTrace(System.out);
        }
    }

}
