package Server.test;

import junit.framework.TestCase;
import Server.db.IDBManager;
import Server.db.DBFactory;
import Server.Plane;
import Server.ITower;


/**
 * Created by IntelliJ IDEA.
 * User: gbylenok
 * Date: Mar 8, 2003
 * Time: 6:17:50 PM
 * To change this template use Options | File Templates.
 */
public class TestDB extends TestCase {

    protected void setUp() throws Exception {
        super.setUp();
    }


    public void testTowers() throws Exception {
        IDBManager db = DBFactory.getManager();

        ITower[] towers = db.loadTowers();

        for (int i = 0; i < towers.length; i++) {
            ITower tower = towers[i];
            System.out.println("Loaded tower:" + tower.getName());
        }

    }

    public void testAllPlaneFunctions() throws Exception {
        IDBManager db = DBFactory.getManager();

        //plane joins the airspace
        int newPlane = db.joinPlane("test-routine");

        try {
            //check to make sure that we cannot select the plane, since it doesn't have a position
            Plane[] somePositions = db.loadPositions();
            for (int i = 0; i < somePositions.length; i++) {
                Plane pos = somePositions[i];

                if (pos.getId() == newPlane)
                        assertEquals(false,pos.isActive() );
            }

            //now give it a position
            Plane record = new Plane(newPlane, 1.5f, 1.6f, 1.7f, 1.8f);
            db.savePosition(record);

            //now see if we can load the plane from the database
            somePositions = db.loadPositions();
            boolean foundPlane = false;
            for (int i = 0; i < somePositions.length; i++) {
                Plane pos = somePositions[i];

                if ((pos.getId() == newPlane) && ( pos.isActive() == true)){
                    foundPlane = true;
                }
            }
            if (foundPlane == false) {
                fail("Couldn't find test plane");
            }
        } finally {
            //be sure to delete the plane from the database now that we are done with the test
            db.removePlane(newPlane);
        }
    }

    /**
     * This tests that a repeat join works as expected
     * @throws Exception
     */
    public void testRepeatJoin() throws Exception {
            IDBManager db = DBFactory.getManager();

        String planeName = "test-repeat-join";

        //plane joins the airspace
        int newPlane = db.joinPlane(planeName);

        //try again
        try
        {
            int anotherPlane = db.joinPlane(planeName);

            if (anotherPlane != newPlane)
            {
                fail("Something wrong with repeat joins.");
            }
        } finally {
            //be sure to delete the plane from the database now that we are done with the test
            db.removePlane(newPlane);
        }

    }

    protected void tearDown() throws Exception {
        super.tearDown();
    }

}
