package Server.test;

import junit.framework.TestCase;
import org.omg.CORBA.ORB;
import org.omg.CosNaming.NamingContextExt;
import org.omg.CosNaming.NamingContextExtHelper;
import ServerModule.TrafficControlHelper;
import ServerModule.TrafficControl;


public class TestExceptions extends TestCase {


    TrafficControl tfImpl;
    private int planeID;

    public TestExceptions() throws Exception{
        super();
        setupORB();
    }

            /**
         * Sets up the CORBA stuff
         * @throws Exception - any exception
         */
        public void setupORB() throws Exception {
            // create and initialize the ORB
            String[] args = new String[]{"-ORBInitialPort", "1500"};
            ORB orb = ORB.init(args, null);

            // get the root naming context
            org.omg.CORBA.Object objRef = orb.resolve_initial_references("NameService");

            // Use NamingContextExt instead of NamingContext. This is
            // part of the Interoperable naming Service.
            NamingContextExt ncRef = NamingContextExtHelper.narrow(objRef);

            // resolve the Object Reference in Naming
            String name = "TrafficControlServer";
            tfImpl = TrafficControlHelper.narrow(ncRef.resolve_str(name));

        }

    protected void setUp() throws Exception {
        super.setUp();

        //register plane
        planeID = tfImpl.Join("plane_setup");
    }

    /**
     * This test assumes a (0,0) to (1,1) coordinate space
     */
    public void testPositionUpdateInvalidCoordinates()
    {
        float bigX = 1.1f;
        float bigY = 1.1f;
        try {
            tfImpl.PositionUpdate(planeID, bigX, bigY, 0f, 0f);
            fail("Expected an invalid_coordinate exception");
        } catch (ServerModule.invalid_plane_id invalid_plane_id) {
            fail("Unexpected Invalid plane ID exception for plane ."+planeID);
        } catch (ServerModule.invalid_coordinate invalid_coordinate) {
            //This is expected, so do nothing
        }
    }

       /**
         * This test assumes a (0,0) to (1,1) coordinate space
         */
        public void testInquiryInvalidCoordinates()
        {
            float bigX = 1.1f;
            float bigY = 1.1f;
            try {
                    tfImpl.Inquiry(planeID, bigX, bigY, 0.1f);
                    fail("Expected an invalid_coordinate exception");
            } catch (ServerModule.system_overload system_overload) {
                fail("Unexpected system overload exception for plane ."+planeID);
            } catch (ServerModule.invalid_plane_id invalid_plane_id) {
                fail("Unexpected Invalid plane ID exception for plane ."+planeID);
            } catch (ServerModule.invalid_coordinate invalid_coordinate) {
                //This is expected, so do nothing
            }
        }


        /**
         * This test assumes a (0,0) to (1,1) coordinate space
         */
        public void testEmergencyInquiryInvalidCoordinates()
        {
            float bigX = 1.1f;
            float bigY = 1.1f;
            try {
                tfImpl.EmergencyInquiry(planeID, bigX, bigY, 0.1f);
                fail("Expected an invalid_coordinate exception");
            } catch (ServerModule.invalid_plane_id invalid_plane_id) {
                fail("Unexpected Invalid plane ID exception for plane ."+planeID);
            } catch (ServerModule.invalid_coordinate invalid_coordinate) {
                //This is expected, so do nothing
            }
        }


    protected void tearDown() throws Exception {
        super.tearDown();
        tfImpl.Leave(planeID);
    }

}
