package Server.test;

import bp.MultiThreadedTestCase;
import org.omg.CORBA.ORB;
import org.omg.CosNaming.NamingContextExt;
import org.omg.CosNaming.NamingContextExtHelper;
import ServerModule.TrafficControlHelper;
import ServerModule.TrafficControl;



public class TestJoinAndLeave extends MultiThreadedTestCase {

    /**
     * Basic constructor - called by the test runners.
     */
    public TestJoinAndLeave(String s) {
        super(s);
    }

    public void test1() {
        TestCaseRunnable tct [] = new TestCaseRunnable[1];
        for (int i = 0; i < tct.length; i++) {
            tct[i] = new TestClientThread();
        }

        runTestCaseRunnables(tct);
    }

    protected class TestClientThread extends MultiThreadedTestCase.TestCaseRunnable {

        TrafficControl tfImpl;

        int maxClients = 10;

        /**
         * Sets up the CORBA stuff
         * @throws Exception - any exception
         */
        public void setupORB() throws Exception {
            // create and initialize the ORB
            String[] args = new String[]{"-ORBInitialPort", "1500"};
            ORB orb = ORB.init(args, null);

            // get the root naming context
            org.omg.CORBA.Object objRef = orb.resolve_initial_references("NameService");

            // Use NamingContextExt instead of NamingContext. This is
            // part of the Interoperable naming Service.
            NamingContextExt ncRef = NamingContextExtHelper.narrow(objRef);

            // resolve the Object Reference in Naming
            String name = "TrafficControlServer";
            tfImpl = TrafficControlHelper.narrow(ncRef.resolve_str(name));

        }


        /**
         * This test assumes that plane IDs are assigned sequentially from
         * 0 up to some max integer (defined in maxClients).
         * @throws Throwable
         */
        public void runTestCase() throws Throwable {

            setupORB();


            int joinLog[] = new int[maxClients +1]; //+1 here so we can overload the server

            //first, prepare the server by unregistering all the planes //////////////
            for (int i = 0; i <= maxClients; i++) {
                try {
                    tfImpl.Leave(i);
                } catch (ServerModule.invalid_plane_id e) {
                    //Ignore. We except this to be thrown once the server is cleared out.
                }
            }

            // next, join a bunch of planes //////////////////////////

            for (int i = 0; i <= maxClients; i++) {

                try {

                    int id = tfImpl.Join("plane"+i);
                    joinLog[i] = id;
                    System.out.println("Client joined with ID: " + id);
                    if (i == maxClients)
                            MultiThreadedTestCase.fail("Server should have reached max number of clients by now. "
                            + "Apparently, it can still accept more.");

                } catch (ServerModule.system_overload system_overload) {
                    //do nothing. This is excepted. Continue...
                }
            }
            //next, each plane leaves and another joins. /////////////////////

            for (int i = 0; i <= maxClients; i++) {
                tfImpl.Leave(joinLog[i]);
                System.out.println("Client left with ID: " + joinLog[i]);

                //server is one under max capacity
                try {
                    joinLog[i] = tfImpl.Join("plane"+i);
                    System.out.println("Client joined with ID: " + joinLog[i]);
                } catch (ServerModule.system_overload system_overload) {
                    MultiThreadedTestCase.fail("Server unexpectedly reached maximum capacity. ");
                }
            }



            //next, plane leaves and tries to join twice /////////////////////
            for (int i = 0; i < maxClients; i++) {
                tfImpl.Leave(joinLog[i]);
                System.out.println("Client left with ID: " + joinLog[i]);

                //server is now one under max capacity
                try {
                    joinLog[i] = tfImpl.Join("plane"+i);
                    System.out.println("Client joined with ID: " + joinLog[i]);
                } catch (ServerModule.system_overload system_overload) {
                    MultiThreadedTestCase.fail("Server unexpectedly reached maximum capacity. ");
                }

                //server is now at max capacity
                try {
                    joinLog[i] = tfImpl.Join("plane"+i);
                    System.out.println("Client joined with ID: " + joinLog[i]);

                    MultiThreadedTestCase.fail("Server should have reached max number of clients by now. "
                            + "Apparently, it can still accept more.");

                } catch (ServerModule.system_overload system_overload) {
                    //do nothing. This is excepted. continue...
                }

            }

            //bring server down to zero
            for (int i = 0; i < maxClients; i++) {

                   tfImpl.Leave(joinLog[i]);

                   // Leave again. an invalid plane exception should be thrown.
                   try
                   {
                    tfImpl.Leave(joinLog[i]);
                       MultiThreadedTestCase.fail("Tried to unregister an unregistered plane. This should have failed. ");
                   }
                    catch (ServerModule.invalid_plane_id e)
                    {
                        assertTrue(true);//Ignore. This is expected.
                   }
                   System.out.println("Client left with ID: " + joinLog[i]);

            }

        }

    }
}